@@ -15,7 +15,7 @@ import copy
 import collections
 
 LeagueStatics = collections.namedtuple("LeagueStatics",
-                                       "pos ir_spots bn_spots settings cats")
+                                       "pos ir_spots bn_spots settings cats ir_name")
 
 
 class ScoreComparer:
@@ -71,12 +71,6 @@ class ScoreComparer:
             stddev_score += v
         return stddev_score
 
-    def print_stdev(self):
-        print("Standard deviations for each category:")
-        for cat, val in self.stdevs.iteritems():
-            print("{} - {:.3f}".format(cat, val.iloc(0)[0]))
-        print("")
-
     def _compute_agg(self, lineups, agg):
         """
         Compute an aggregation of each of the categories
@@ -123,7 +117,6 @@ class ManagerBot:
         self.scorer = Scorer(self.cfg)
         Display = self._get_display_class()
         self.display = Display(self.cfg)
-        self.blacklist = self._load_blacklist()
         self.lineup = None
         self.bench = []
         self.injury_reserve = []
@@ -136,16 +129,7 @@ class ManagerBot:
         self.fetch_player_pool()
         self.sync_lineup()
         self.pick_injury_reserve()
-        self.auto_pick_opponent()
-
-    def _load_blacklist(self):
-        fn = self.tm_cache.blacklist_cache_file()
-        if os.path.exists(fn):
-            with open(fn, "rb") as f:
-                blacklist = pickle.load(f)
-        else:
-            blacklist = []
-        return blacklist
+        self._auto_pick_opponent()
 
     def pick_bench(self):
         """Pick the bench spots based on the current roster."""
@@ -178,7 +162,7 @@ class ManagerBot:
         ir = []
         roster = self._get_orig_roster()
         for plyr in roster:
-            if plyr['status'] == 'IR':
+            if plyr['status'].startswith(self.lg_statics.ir_name):
                 ir.append(plyr)
                 for idx, lp in enumerate(self.lineup):
                     if lp['player_id'] == plyr['player_id']:
@@ -189,7 +173,7 @@ class ManagerBot:
                         del self.bench[idx]
                         break
 
-        if len(ir) < self.lg_statics.ir_spots:
+        if len(ir) <= self.lg_statics.ir_spots:
             self.injury_reserve = ir
         else:
             assert(False), "Need to implement pruning of IR"
@@ -212,25 +196,27 @@ class ManagerBot:
                         del self.lineup[idx]
                         break
 
-    def _save_blacklist(self):
-        fn = self.tm_cache.blacklist_cache_file()
-        with open(fn, "wb") as f:
-            pickle.dump(self.blacklist, f)
+    def move_recovered_il_to_bench(self):
+        """Move any player that is currently on the IL but no longer eligible
 
-    def add_to_blacklist(self, plyr_name):
-        self.blacklist.append(plyr_name)
-        self._save_blacklist()
-
-    def remove_from_blacklist(self, plyr_name):
-        if plyr_name not in self.blacklist:
-            return False
-        else:
-            self.blacklist.remove(plyr_name)
-            self._save_blacklist()
-            return True
-
-    def get_blacklist(self):
-        return self.blacklist
+        Any player is moved to the bench.  This is temporary and may cause the size of the bench to increase the max.
+        We keep them at the bench so they can be used in the lineup (if needed).  The bench is fixed up when
+        pick_bench() is called.
+        """
+        roster = self._get_orig_roster()
+        for plyr in roster:
+            assert(plyr['selected_position'] != 'IR')  # Need to account for IR
+            if plyr['selected_position'] == 'IL' and 'IL' not in plyr['eligible_positions']:
+                plyr_from_pool = self.ppool[self.ppool['player_id'] == plyr['player_id']]
+                self.bench.append(plyr_from_pool.iloc(0)[0])
+                for idx, lp in enumerate(self.lineup):
+                    if lp['player_id'] == plyr['player_id']:
+                        self.logger.info(
+                            "Moving {} out of the IL slot because "
+                            "they are no longer eligible ({})".format(
+                                plyr['name'], plyr['eligible_positions']))
+                        del self.lineup[idx]
+                        break
 
     def load_league_statics(self):
         """Load static settings for the league.
@@ -244,10 +230,13 @@ class ManagerBot:
             pos = self.lg.positions()
             if "IR" in pos:
                 ir_spots = pos['IR']['count']
+                ir_name = 'IR'
             elif "IL" in pos:
                 ir_spots = pos['IL']['count']
+                ir_name = 'IL'
             else:
                 ir_spots = 0
+                ir_name = None
             bn_spots = pos['BN']['count'] if "BN" in pos else 0
             for del_pos in ['IR', 'IL', 'BN']:
                 if del_pos in pos:
@@ -256,7 +245,8 @@ class ManagerBot:
                                  ir_spots=ir_spots,
                                  bn_spots=bn_spots,
                                  settings=self.lg.settings(),
-                                 cats=self.lg.stat_categories())
+                                 cats=self.lg.stat_categories(),
+                                 ir_name=ir_name)
         self.lg_statics = self.lg_cache.load_statics(loader)
 
     def init_prediction_builder(self):
@@ -271,16 +261,12 @@ class ManagerBot:
             minutes=int(self.cfg['Cache']['predictionBuilderExpiry']))
         self.pred_bldr = self.tm_cache.load_prediction_builder(expiry, loader)
 
-    def save(self):
-        self.tm_cache.refresh_prediction_builder(self.pred_bldr)
-
     def fetch_cur_lineup(self):
         """Fetch the current lineup as set in Yahoo!"""
         all_mine = self._get_orig_roster()
         pct_owned = self.lg.percent_owned([e['player_id'] for e in all_mine])
         for p, pct_own in zip(all_mine, pct_owned):
-            if p['selected_position'] == 'BN' or \
-                    p['selected_position'] == 'IR':
+            if p['selected_position'] in ['BN', self.lg_statics.ir_name]:
                 p['selected_position'] = np.nan
             assert(pct_own['player_id'] == p['player_id'])
             p['percent_owned'] = pct_own['percent_owned']
@@ -334,7 +320,7 @@ class ManagerBot:
             with open(self.tm_cache.free_agents_cache_file(), "wb") as f:
                 pickle.dump(free_agents, f)
 
-    def sum_opponent(self, opp_team_key):
+    def _sum_opponent(self, opp_team_key):
         # Build up the predicted score of the opponent
         try:
             team_name = self._get_team_name(self.lg, opp_team_key)
@@ -356,7 +342,8 @@ class ManagerBot:
         for plyr in self.bench:
             if plyr["player_id"] not in new_plyr_ids:
                 new_bench.append(plyr)
-        assert(len(new_bench) <= self.lg_statics.bn_spots)
+        # Note, it is possible for new_bench to be greater than the allowed.  We allow this
+        # temporarily as the bench will be trimmed later.
         self.lineup = new_lineup
         self.bench = new_bench
 
@@ -424,8 +411,7 @@ class ManagerBot:
         bench_ids = [e['player_id'] for e in yahoo_roster
                      if e['selected_position'] == 'BN']
         ir_ids = [e['player_id'] for e in yahoo_roster
-                  if (e['selected_position'] == 'DL' or
-                      e['selected_position'] == 'IR')]
+                  if (e['selected_position'] == self.lg_statics.ir_name)]
         lineup = []
         bench = []
         ir = []
@@ -447,10 +433,19 @@ class ManagerBot:
         :return: Player pool
         :rtype: DataFrame
         """
-        avail_plyrs = self.ppool[~self.ppool['name'].isin(self.blacklist)]
-        avail_plyrs = avail_plyrs[avail_plyrs['percent_owned'] > 10]
+        avail_plyrs = self.ppool[self.ppool['percent_owned'] > 10]
         return avail_plyrs[avail_plyrs['status'] == '']
 
+    def _get_locked_players_list(self):
+        locked_file = self.cfg['LineupOptimizer']['lockPlayerFile']
+        if locked_file != "":
+            if not os.path.isfile(locked_file):
+                raise RuntimeError("Could not open locked file: {}".format(locked_file))
+            with open(locked_file) as f:
+                return [p.strip() for p in f.readlines()]
+        else:
+            return []
+
     def optimize_lineup_from_free_agents(self):
         """
         Optimize your lineup using all of your players plus free agents
@@ -460,12 +455,14 @@ class ManagerBot:
         optimizer_func = self._get_lineup_optimizer_function()
 
         locked_plyrs = []
+        locked_from_file = self._get_locked_players_list()
         thres = int(self.cfg['LineupOptimizer']['lockPlayersAbovePctOwn'])
         for plyr in self.lineup:
-            if plyr['percent_owned'] >= thres:
+            if plyr['percent_owned'] >= thres or plyr['name'] in locked_from_file:
                 clone_plyr = copy.deepcopy(plyr)
                 clone_plyr['selected_position'] = np.nan
                 locked_plyrs.append(clone_plyr)
+                self.logger.info("{} is added to locked list ({}% owned)".format(plyr['name'], plyr['percent_owned']))
 
         best_lineup = optimizer_func(self.cfg, self.score_comparer,
                                      self.my_team_bldr,
@@ -474,87 +471,6 @@ class ManagerBot:
             self.lineup = copy.deepcopy(best_lineup.get_roster())
         return best_lineup is not None
 
-    def show_score(self):
-        if self.opp_sum is None:
-            raise RuntimeError("No opponent selected")
-
-        self.score_comparer.print_stdev()
-
-        df = pd.DataFrame(data=self.lineup, columns=self.lineup[0].index)
-        my_sum = self.scorer.summarize(df)
-        score = self.score_comparer.compute_score(my_sum)
-        print("Against '{}' your roster has a score of: {}".
-              format(self.opp_team_name, score))
-        print("")
-        for stat in my_sum.keys():
-            if stat in ["ERA", "WHIP"]:
-                if math.isclose(my_sum[stat], self.opp_sum[stat]):
-                    my_win = "="
-                    opp_win = "="
-                elif my_sum[stat] < self.opp_sum[stat]:
-                    my_win = "*"
-                    opp_win = ""
-                else:
-                    my_win = ""
-                    opp_win = "*"
-            else:
-                if math.isclose(my_sum[stat], self.opp_sum[stat]):
-                    my_win = "="
-                    opp_win = "="
-                elif my_sum[stat] > self.opp_sum[stat]:
-                    my_win = "*"
-                    opp_win = ""
-                else:
-                    my_win = ""
-                    opp_win = "*"
-            print("{:5} {:2.3f} {:1} v.s. {:2.3f} {:2}".format(
-                stat, my_sum[stat], my_win, self.opp_sum[stat], opp_win))
-
-    def list_players(self, pos):
-        self.display.printListPlayerHeading(pos)
-
-        for plyr in self.ppool.iterrows():
-            if pos in plyr[1]['eligible_positions']:
-                self.display.printPlayer(pos, plyr)
-
-    def find_in_lineup(self, name):
-        for idx, p in enumerate(self.lineup):
-            if p['name'] == name:
-                return idx
-        raise LookupError("Could not find player: " + name)
-
-    def swap_player(self, plyr_name_del, plyr_name_add):
-        if plyr_name_add:
-            plyr_add_df = self.ppool[self.ppool['name'] == plyr_name_add]
-            if(len(plyr_add_df.index) == 0):
-                raise LookupError("Could not find player in pool: {}".format(
-                    plyr_name_add))
-            if(len(plyr_add_df.index) > 1):
-                raise LookupError("Found more than one player!: {}".format(
-                    plyr_name_add))
-            plyr_add = plyr_add_df.iloc(0)[0]
-        else:
-            plyr_add = None
-
-        idx = self.find_in_lineup(plyr_name_del)
-        plyr_del = self.lineup[idx]
-        assert(type(plyr_del.selected_position) == str)
-        if plyr_add and plyr_del.selected_position not in \
-                plyr_add['eligible_positions']:
-            raise ValueError("Position {} is not a valid position for {}: {}".
-                             format(plyr_del.selected_position,
-                                    plyr_add['name'],
-                                    plyr_add['eligible_positions']))
-
-        if plyr_add:
-            plyr_add['selected_position'] = plyr_del['selected_position']
-        plyr_del['selected_position'] = np.nan
-        if plyr_add:
-            self.lineup[idx] = plyr_add
-        else:
-            del(self.lineup[idx])
-        self.pick_bench()
-
     def apply_roster_moves(self, dry_run, prompt):
         """Make roster changes with Yahoo!
 
@@ -564,7 +480,7 @@ class ManagerBot:
         """
         roster_chg = RosterChanger(self.lg, dry_run, self._get_orig_roster(),
                                    self.lineup, self.bench,
-                                   self.injury_reserve, prompt)
+                                   self.injury_reserve, self.lg_statics.ir_name, prompt)
         roster_chg.apply()
 
         # Change the free agent cache to remove the players we added
@@ -572,11 +488,11 @@ class ManagerBot:
             adds = roster_chg.get_adds_completed()
             self.invalidate_free_agents(adds)
 
-    def pick_opponent(self, opp_team_key):
-        (self.opp_team_name, self.opp_sum) = self.sum_opponent(opp_team_key)
+    def _pick_opponent(self, opp_team_key):
+        (self.opp_team_name, self.opp_sum) = self._sum_opponent(opp_team_key)
         self.score_comparer.set_opponent(self.opp_sum)
 
-    def auto_pick_opponent(self):
+    def _auto_pick_opponent(self):
         edit_wk = self.lg.current_week()
         (wk_start, wk_end) = self.lg.week_date_range(edit_wk)
         edit_date = self.lg.edit_date()
@@ -589,7 +505,7 @@ class ManagerBot:
             self.logger.info("Could not find opponent.  Picking ourselves...")
             opp_team_key = self.lg.team_key()
 
-        self.pick_opponent(opp_team_key)
+        self._pick_opponent(opp_team_key)
 
     def evaluate_trades(self, dry_run, verbose, prompt=False):
         """
@@ -693,25 +609,13 @@ class ManagerBot:
                 pos_list.append(pos_name)
         return roster.Builder(pos_list)
 
-    def _get_position_types(self):
-        settings = self.lg.settings()
-        position_types = {'mlb': ['B', 'P'], 'nhl': ['G', 'P']}
-        return position_types[settings['game_code']]
-
-    def _is_predicted_stat(self, stat):
-        return stat in self.cfg['League']['predictedStatCategories'].split(',')
-
     def _get_orig_roster(self):
         return self.lg.to_team(self.lg.team_key()).roster(
             day=self.lg.edit_date())
 
     def _call_predict(self, plyrs, fail_on_missing):
-            kwargs = {
-                k: v for k, v in self.cfg['PredictionNamedArguments'].items()
-            }
-            return self.pred_bldr.predict(
-                plyrs, fail_on_missing=fail_on_missing,
-                **kwargs)
+            return self.pred_bldr.predict(plyrs,
+                                          fail_on_missing=fail_on_missing)
 
     def _get_roster_for_team(self, team):
         """Get all the players that are active for a given team
@@ -731,7 +635,7 @@ class ManagerBot:
 
 class RosterChanger:
     def __init__(self, lg, dry_run, orig_roster, lineup, bench,
-                 injury_reserve, prompt):
+                 injury_reserve, ir_name, prompt):
         self.lg = lg
         self.tm = lg.to_team(lg.team_key())
         self.dry_run = dry_run
@@ -740,6 +644,7 @@ class RosterChanger:
         self.lineup = lineup
         self.bench = bench
         self.injury_reserve = injury_reserve
+        self.ir_name = ir_name
         self.orig_roster_ids = [e['player_id'] for e in orig_roster]
         self.new_roster_ids = [e['player_id'] for e in lineup] + \
             [e['player_id'] for e in bench] + \
@@ -826,7 +731,7 @@ class RosterChanger:
 
     def _apply_ir_moves(self):
         orig_ir = [e for e in self.orig_roster
-                   if e['selected_position'] == 'IR']
+                   if e['selected_position'] == self.ir_name]
         new_ir_ids = [e['player_id'] for e in self.injury_reserve]
         pos_change = []
         num_drops = 0
@@ -841,7 +746,7 @@ class RosterChanger:
         for plyr in self.injury_reserve:
             assert(plyr['player_id'] in self.orig_roster_ids)
             pos_change.append({'player_id': plyr['player_id'],
-                               'selected_position': 'IR',
+                               'selected_position': self.ir_name,
                                'name': plyr['name']})
             num_drops -= 1
 
